import { CustomEvent } from "../custom.event";

type PositionEx = {
  x: number;
  y: number;
  z: number;
};

const localPlayer = mp.players.local;

class MarkerManager {
  private marker: MarkerMp | null = null;
  private renderHandler: (() => void) | null = null;
  private currentColor: [number, number, number, number] = [255, 0, 0, 255];
  private currentScale: number = 0.7;
  private scaleDecreaseRate: number = 0.01;

  public createTreasureRender(position: PositionEx, radius: number) {
    if (this.marker) return;

    const initPosition = {
      x: localPlayer.position.x,
      y: localPlayer.position.y,
      z: localPlayer.position.z - 1,
    };
    this.marker = mp.markers.new(
      27,
      initPosition as Vector3Mp,
      this.currentScale,
      {
        color: this.currentColor,
        visible: true,
        dimension: localPlayer.dimension,
      }
    );

    this.renderHandler = () => {
      if (!this.marker) return;

      const forwardVector = this.getForwardVector();
      const markerPosition = {
        x: forwardVector.x,
        y: forwardVector.y,
        z: localPlayer.position.z - 0.7,
      };

      this.marker.position = markerPosition as Vector3Mp;

      const distance = mp.game.system.vdist(
        localPlayer.position.x,
        localPlayer.position.y,
        localPlayer.position.z,
        position.x,
        position.y,
        position.z
      );

      const maxDecreaseRate = 0.05;
      this.scaleDecreaseRate = (1 - distance / radius) * maxDecreaseRate;

      this.currentScale -= this.scaleDecreaseRate;
      if (this.currentScale <= 0.0) {
        this.currentScale = 0.7;
      }

      this.marker.scale = this.currentScale;

      let newColor: [number, number, number, number];

      if (distance < 2) {
        newColor = [0, 255, 0, 255];
      } else if (distance >= 2 && distance < radius / 2) {
        newColor = [255, 165, 0, 255];
      } else {
        newColor = [255, 0, 0, 255];
      }

      if (this.currentColor.toString() !== newColor.toString()) {
        this.marker.destroy();
        this.currentColor = newColor;
        this.marker = mp.markers.new(
          27,
          initPosition as Vector3Mp,
          this.currentScale,
          {
            color: this.currentColor,
            visible: true,
            dimension: localPlayer.dimension,
          }
        );
      }
    };

    mp.events.add("render", this.renderHandler);
  }

  public removeTreasureRender() {
    if (this.marker) {
      this.marker.destroy();
      this.marker = null;
    }

    if (this.renderHandler) {
      mp.events.remove("render", this.renderHandler);
      this.renderHandler = null;
    }
  }

  private getForwardVector() {
    const rotation = localPlayer.getHeading();
    const radians = (-rotation * Math.PI) / 180;
    const nx = localPlayer.position.x + 1 * Math.sin(radians);
    const ny = localPlayer.position.y + 1 * Math.cos(radians);
    return {
      x: nx,
      y: ny,
      z: 0,
    };
  }
}

const markerManager = new MarkerManager();

CustomEvent.registerServer(
  "treasureHunter:enterBigZone",
  (position: PositionEx, radius: number) => {
    markerManager.createTreasureRender(position, radius);
  }
);

CustomEvent.registerServer("treasureHunter:exitBigZone", () => {
  markerManager.removeTreasureRender();
});
