import { colshapeHandle, colshapes } from "../checkpoints";
import {
	treasurePositions,
	treasureTimer,
	treasureItems,
	treasureColshape,
} from "./config";
import { CustomEvent } from "../custom.event";
import { inventory } from "../inventory";
import { OWNER_TYPES } from "../../../shared/inventory";

class TreasureHunter {
	private treasureColshapes: Map<
		number,
		{ small: colshapeHandle; big: colshapeHandle }
	>;

	constructor() {
		this.treasureColshapes = new Map();
		this.initializeTreasurePositions();
	}

	private initializeTreasurePositions() {
		treasurePositions.forEach((treasure, index) => {
			this.createColshape(treasure, index);
		});
	}

	private createColshape(
		treasure: { x: number; y: number; z: number },
		index: number,
	) {
		const colshapeSmall = colshapes.new(
			new mp.Vector3(treasure.x, treasure.y, treasure.z),
			(player) => (this.getMetalDetectorStatus(player) ? "Выкопать" : ""),
			(player) => this.digging(player, index),
			{
				dimension: 0,
				radius: 2,
				type: -1,
			},
		);
		const colshapeBig = colshapes.new(
			new mp.Vector3(treasure.x, treasure.y, treasure.z),
			null,
			null,
			{
				dimension: 0,
				radius: treasureColshape,
				onEnterHandler: (player) => this.onEnterBigZone(player, index),
				onExitHandler: (player) => this.onExitBigZone(player, index),
				type: -1,
			},
		);
		const colshapesData = {
			small: colshapeSmall,
			big: colshapeBig,
		};
		this.treasureColshapes.set(index, colshapesData);
	}

	private onEnterBigZone(player: PlayerMp, index: number) {
		const metalDetectorUse = this.getMetalDetectorStatus(player);
		if (metalDetectorUse) {
			const colshape = this.treasureColshapes.get(index);
			if (colshape) {
				CustomEvent.triggerClient(
					player,
					"treasureHunter:enterBigZone",
					colshape.small.position[0],
					treasureColshape,
				);
			}
		}
	}

	private onExitBigZone(player: PlayerMp, index: number) {
		const metalDetectorUse = this.getMetalDetectorStatus(player);
		if (metalDetectorUse) {
			const colshape = this.treasureColshapes.get(index);
			if (colshape) {
				CustomEvent.triggerClient(player, "treasureHunter:exitBigZone");
			}
		}
	}

	private digging(player: PlayerMp, index: number) {
		player.user.removeAttachment("metaldetector");
		player.stopAnimation();
		setTimeout(() => {
			player.user.playAnimation(
				[["mini@golf", "iron_idle_high_a"]],
				false,
				true,
			);
		}, 500);

		this.removeTreasureColshape(index);
		this.toggleMetalDetector(player);
		CustomEvent.triggerClient(player, "treasureHunter:exitBigZone");
		setTimeout(() => {
			this.giveTreasure(player);
			player.stopAnimation();
		}, 4000);
	}

	private giveTreasure(player: PlayerMp) {
		const treasure = this.getRandomTreasureItem();
		if (treasure) {
			inventory.createItem({
				item_id: treasure.id,
				owner_type: OWNER_TYPES.PLAYER,
				owner_id: player.user.id,
			});
			player.notify(`Вы нашли сокровище! Предмет: ${treasure.name}`, "success");
		} else {
			player.notify("В этот раз ничего не найдено.", "info");
		}
	}

	private getRandomTreasureItem() {
		const totalChance = treasureItems.reduce(
			(sum, item) => sum + item.chance,
			0,
		);
		const random = Math.random() * totalChance;

		let accumulatedChance = 0;
		for (const item of treasureItems) {
			accumulatedChance += item.chance;
			if (random <= accumulatedChance) {
				return item;
			}
		}
		return null;
	}
	private removeTreasureColshape(index: number) {
		const treasure = this.treasureColshapes.get(index);
		if (treasure) {
			treasure.small.destroy();
			treasure.big.destroy();
			this.treasureColshapes.delete(index);

			setTimeout(
				() => {
					const treasurePosition = treasurePositions[index];
					this.createColshape(treasurePosition, index);
				},
				treasureTimer * 60 * 1000,
			);
		}
	}

	private getMetalDetectorStatus(player: PlayerMp): boolean {
		return !!player.getVariable("metalDetectorUse");
	}

	public toggleMetalDetector(player: PlayerMp) {
		const metalDetectorUse = player.getVariable("metalDetectorUse");
		if (metalDetectorUse) {
			player.setVariable("metalDetectorUse", false);
			player.stopAnimation();
			player.user.removeAttachment("metaldetector");
		} else {
			player.setVariable("metalDetectorUse", true);
			player.user.addAttachment("metaldetector");
			player.user.playAnimation(
				[["timetable@gardener@clean_pool@", "idle_a_gardener"]],
				true,
				true,
			);
		}
	}
}

const treasureHunter = new TreasureHunter();
export default treasureHunter;
